# Map census data 
#'
#' @param census_data, dataframe of census data for specified variable of interest 
#' @param conus_sf, sf of conus states outline
#' @param leg_title, character string for legend title
#' @param outfile_path, outfile path for pngs 
#' @param viz_config_df `data.frame` width, height, counties outline color, conus outline color, background color, font nam, and font size 
#' @param viz_config_pal `data.frame` assign colors for postively and negatively correlated dimensions for census maps
#' @param width, set figure width dimension
#' @param height, set figure height dimension
#' @param percent_leg if else statement where if TRUE, apply 0-100 legend, otherwise retain 0 - max of variable name
#' @param dollar_leg if else statement where if TRUE, apply $ to the estimate column to display cost in USD on map
#' @param font_size, set font size 
#' @param barheight, set colorbar bar height
#' @param barwidth, set colorbar bar width 
#' @param lim_vals, set legend limits
#' @param break_vals, set legend breaks 
#' @param low_ramp_col, assign color to low end of color ramp for legend 
plot_census_map <- function(census_data, conus_sf, leg_title, outfile_path, var, 
                            percent_leg, viz_config_df, viz_config_pal, width, height,
                            font_size, barwidth, barheight, lim_vals, break_vals,
                            dollar_leg, low_ramp_col){
  
  font_legend <- viz_config_df$load_font
  font_add_google(font_legend)
  showtext_opts(dpi = 300, regular.wt = 200, bold.wt = 700)
  showtext_auto(enable = TRUE)
  
  census_map <- census_data |> 
    ggplot(aes(fill = .data[[var]])) + 
    geom_sf(color = viz_config_df$counties_outline_col,
            linewidth = 0.05) +
    geom_sf(data = conus_sf,
            fill = NA,
            color = viz_config_df$conus_outline_col,
            linewidth = 0.5,
            linetype = "solid") + 
    theme_void() +
    theme(text = element_text(family = viz_config_df$font_legend, size = font_size),
          legend.margin = margin(t = 5, b = 5), legend.position = 'bottom', legend.title.align = 0.5) +
    guides(fill = guide_colorbar(
           title.position = "top",
           title.theme = element_text(face = 'bold', family = viz_config_df$font_legend, size = font_size),
           direction = "horizontal",
           position = "bottom",
           barwidth = barwidth, 
           barheight = barheight 
           )) 
  
  if (percent_leg == FALSE) {
    census_map <- census_map +     
      scale_fill_gradientn(
        colors = colorRampPalette(c(low_ramp_col, viz_config_pal))(100), 
        name = leg_title,
        limits = lim_vals,
        labels = if (dollar_leg) scales::label_dollar() else scales::comma,
        na.value = "#F5F5F5"
      ) 
  
 } else {
   census_map <- census_map +     
     scale_fill_gradientn(
       colors = colorRampPalette(c("#F8F9FF", viz_config_pal))(100), 
       name = leg_title,
       limits = lim_vals,  # c(0, 100),
       breaks = break_vals, # c(0, 25, 50, 75, 100),
       labels = if (dollar_leg) scales::label_dollar() else function(x) paste0(x, "%"),
       na.value="#F5F5F5"
     )
 }

  
  background_color = "white"
  plot_margin =  0.025
  
  # cowplot to get map sizes larger
  canvas <- grid::rectGrob(
    x = 0, y = 0,
    width = width, height = height,
    gp = grid::gpar(fill = background_color, alpha = 1, col = background_color
    )
  )
      census_legend <-  get_plot_component(census_map, 'guide-box', return_all = TRUE)
    # compose final plot
    final_map <- ggdraw(ylim = c(0,1),
                        xlim = c(0,1)) +
      # White background
      draw_grob(canvas,
                x = 0, y = 1,
                height = height, width = width,
                hjust = 0, vjust = 1) +
      # Add main plot
      draw_plot(census_map + theme(legend.position="none"),
                x = -0.01,
                y = 0.08,
                height = 0.98,
                width = (1-plot_margin)*1.03) +
      # Add legend 
      draw_plot(census_legend[[3]],
                x = 0.48,
                y = 0.02,
                height = 0.09 ,
                width = 0.1 - plot_margin) 
  
ggsave(outfile_path, final_map, width = width, height = height, dpi = viz_config_df$dpi, bg = viz_config_df$bg_col, units = "in")
  
  return(outfile_path)
}